#ifndef __ALLORDERGENERATORFLAT_HH__
#define __ALLORDERGENERATORFLAT_HH__

#include "AllOrderGenerator.hh"
 
/// Calculate the successive splittings of an initial parton.
class AllOrderGeneratorFlat : public AllOrderGenerator {
public:
  AllOrderGeneratorFlat(int seed, Parton::Flavour flav, double eps, double t)
    : AllOrderGenerator(seed,flav,eps,t) {}

  AllOrderGeneratorFlat(int seed, Parton::Flavour flav, double eps, double t,
		    double ca, double cf, double tr, double nf)
    : AllOrderGenerator(seed,flav,eps,t,ca,cf,tr,nf) {}

  ~AllOrderGeneratorFlat() {}

  /// Run evolution of _events.back() (from initial state t0 to final state tmax)
  void next_event_group() {
    // carry out splittings until we reach tmax
    while (_t < _tmax) {
      split();
    }
    return;
  }

private:

  ///Split a particle randomly according to weights
  void split(){ 
    int n=_events.back().size(),index=0;
    Parton::Flavour fla1=Parton::q,fla2=Parton::q;
    double tot=0,tmp=0,r,z;
    r = _rng.uniform();
    std::vector<double> wpqq(n,0),wpqg(n,0),wpgg(n,0);
    for(int i=0;i<n;++i) {
      if (_events.back()[i].flav()==Parton::q) {
	wpqq[i]=weight_pqq(_events.back()[i].x());
	tot+=wpqq[i];      
      }
      if (_events.back()[i].flav()==Parton::g) {
	wpqg[i]=weight_pqg(_events.back()[i].x());
	wpgg[i]=weight_pgg(_events.back()[i].x());
	tot+=wpqg[i]+wpgg[i];
      }
    }

    // determine new t value, and decide if we continue
    _t += _rng.exponential(1.0/tot);
    if (_t > _tmax) {
      _t = _tmax;
      return;
    }

    // for each particle, find the one that splits
    z = -1; //we can check at the end that it is set
    for(int i=0;i<n;++i){
      if (r*tot<(tmp+=wpqq[i])) {
	index=i;
	fla1=Parton::q;
	fla2=Parton::g;
	z=split_pqq(_events.back()[i].x());
	break;
      } else if (r*tot<(tmp+=wpqg[i])) {
	index=i;
	fla1=Parton::q;
	fla2=Parton::q;
	z=split_pqg(_events.back()[i].x());
	break;
      } else if(r*tot<(tmp+=wpgg[i])) {
	index=i;
	fla1=Parton::g;
	fla2=Parton::g;
	z=split_pgg(_events.back()[i].x());
	break;
      } 
    }
    if (z < 0) {
      std::cerr << "ERROR in split: the code should not have arrived here" << std::endl;
      exit(-1);
    }
    split(index, z, fla1, fla2);
    return;
  }

  /// splitting the particle at index i into x,flav1 & (1-x),flav2
  /// add the second particle at the end of _events.back()
  void split(const int i, const double x, Parton::Flavour flav1, Parton::Flavour flav2) {
    Parton part2(_events.back()[i]);
    part2.scale(1-x);
    part2.flav(flav2);
    _events.back()[i].scale(x);
    _events.back()[i].flav(flav1);
    _events.back().push_back(part2);
  }
  /// q ->qg splitting
  double split_pqq(const double x) {
    double r,pass, z;
    bool done=false;
    while(!done) {
      r = _rng.uniform();
      pass =  _rng.uniform();
      if ((r!=0.) && (r!=1.)) {
	z=z_simplified_pqq(r,x);
	if (pass < (pqq(z)/pqqs(z)))
	  done=true;
      }
    }
    return z;
  }
  /// g -> qq splitting
  double split_pqg(const double x) {
    double r,pass, z;
    bool done=false;
    while(!done) {
      r = _rng.uniform();
      pass = _rng.uniform();
      if ((r!=0.) && (r!=1.)) {
	z=z_simplified_pqg(r,x);
	if (pass < (pqg(z)/pqgs(z)))
	  done=true;
      }
    }
    return z;
  }
  /// g -> gg splitting
  double split_pgg(const double x) {
    double r,pass, z;
    bool done=false;
    while(!done) {
      r = _rng.uniform();
      pass = _rng.uniform();
      if ((r!=0.) && (r!=1.)) {
	z=z_simplified_pgg(r,x);
	if (pass < (pgg(z)/pggs(z)))
	  done=true;
      }
    }
    return z;
  }

  // splitting functions and simplified form
  /// Simplified pqq function with pqqs(x)>pqq(x)
  double pqqs(const double x) {
    return 2/(1-x); }
  /// q -> qg splitting function
  double pqq(const double x) {
    return (1+x*x)/(1-x); }
  /// Simplified pqg function with pqgs(x)>pqg(x)
  double pqgs(const double x) {
    return 1.; }
  /// g -> qq splitting function
  double pqg(const double x) {
    return x*x+(1-x)*(1-x); }
  /// Simplified pgg function with pggs(x)>pgg(x)
  double pggs(const double x) {
    return 1/x+1/(1-x); }
  /// g -> gg splitting function
  double pgg(const double x) {
    return x/(1-x)+(1-x)/x+x*(1-x); }

  /// weight of pqq for splitting
  double weight_pqq(const double x) {
    if (x<2*_eps) return 0;
    double eps(_eps/x);
    return _CF*(-3+6*eps+8*atanh(1-2*eps))/2.;
  }
  /// weight of pqg for splitting
  double weight_pqg(const double x){
    if (x<2*_eps) return 0;
    double eps(_eps/x);
    return _nf*_TR*(2./3.-2*eps+2*eps*eps-4*eps*eps*eps/3.);
  }
  /// weight of pgg for splitting
  double weight_pgg(const double x){
    if (x<2*_eps) return 0;
    double eps(_eps/x);
    return _CA*(-11+24*eps-6*eps*eps+4*eps*eps*eps+24*atanh(1-2*eps))/6.;
  }

  /// z for simplified splitting fct, should be accepted with Pqq(z)/Pqqs(z)
  double z_simplified_pqq(const double r, const double x) {
    if (x<2*_eps) return 0;
    double eps(_eps/x);
    return 1-(1-eps)*exp(r*(log(eps)-log(1-eps)));
  }
  /// z for simplified splitting fct, should be accepted with Pqg(z)/Pqgs(z)
  double z_simplified_pqg(const double r, const double x) {
    if (x<2*_eps) return 0;
    double eps(_eps/x);
    return r*(1-2*eps)+eps;
  }
  /// z for simplified splitting fct, should be accepted with Pgg(z)/Pggs(z)
  double z_simplified_pgg(const double r, const double x) {
    if (x<2*_eps) return 0;
    double eps(_eps/x);
    double delta(exp(2*r*log((1/eps)-1)));
    return delta/(delta+(1-eps)/eps);
  }
};

#endif // __ALLORDERGENERATORFLAT_HH__
