#ifndef __ALLORDERGENERATORRECURSIVE_HH__
#define __ALLORDERGENERATORRECURSIVE_HH__

#include "AllOrderGenerator.hh"
#include <cstdlib>
#include <assert.h>

/// Calculate the evolution of a parton recursively.
class AllOrderGeneratorRecursive : public AllOrderGenerator {
public:
  AllOrderGeneratorRecursive(int seed, Parton::Flavour flav, double eps, double t) 
    : AllOrderGenerator(seed,flav,eps,t) {}
  AllOrderGeneratorRecursive(int seed, Parton::Flavour flav, double eps, double t,
		      double ca, double cf, double tr, double nf) 
    : AllOrderGenerator(seed,flav,eps,t,ca,cf,tr,nf) {}
 
 
  ~AllOrderGeneratorRecursive() {}

  void next_event_group() {
    int n=_events.back().size();
    for(int i=0;i<n; ++i)
      sudakov_recursive_split(i, _t, _tmax, _eps);
    _t=_tmax;
  }

private:
  double Pqq(double x) {return _CF*(1.0 + x*x)/(1.0-x);}
  /// the Pgg splitting function including a combinatoric 1/2 factor
  double Pgg(double x) {return _CA*(x/(1.0-x) + (1.0-x)/x + x*(1.0-x));}
  double Pgg_asym(double x) {return _CA*(2*x/(1.0-x) + x*(1.0-x));}
  double Pqg(double x) {return _nf*_TR*(x*x + (1.0-x)*(1.0-x));}
  
  // simple functions that are always larger than the corresponding
  // splitting functions
  double Pgg_asym_sup(double x) {return _CA*2/(1.0-x);}
  double Pqq_sup     (double x) {return _CF*2/(1.0-x);}
  double Pqg_sup     (double x) {return _nf*_TR;}
  
  // integrals of the above simple functions between two limits x1, x2
  double Pgg_asym_sup_int(double x1, double x2) {return _CA*2.0*std::log((1.0-x1)/(1.0-x2));}
  double Pqq_sup_int     (double x1, double x2) {return _CF*2.0*std::log((1.0-x1)/(1.0-x2));}
  double Pqg_sup_int     (double x1, double x2) {return _nf*_TR*(x2-x1);}

  /// split particle with index i to z(a) (1-z)(b)
  void split(int index, double z, Parton::Flavour flav_a, Parton::Flavour flav_b) {
    _events.back().push_back(Parton((1-z)*_events.back()[index].x(),flav_b));
    _events.back()[index].scale(z);
    _events.back()[index].flav(flav_a);
  }

  // carry out a random splitting of particle with index i
  void split(int index) {
    double z = _rng.uniform_pos();
    if (_events.back()[index].flav() == Parton::q) {
      split(index, z, Parton::q, Parton::g);
    } else {
      double wt_pgg = Pgg(z);
      double wt_pqg = Pqg(z);
      double wt_total = wt_pgg + wt_pqg;
      if (_rng.uniform_pos() * wt_total < wt_pgg ) {
        split(index, z, Parton::g, Parton::g);
      } else {
        split(index, z, Parton::q, Parton::q);
      }
    }
  }
  /// take the quark with the specified index and specified t value,
  /// and split it, updating the t value, and adding the new emission
  /// to the end of our array. The divergences near x=1 (and if
  /// relevant near x=0) are regularised by cutoff epsilon.
  ///
  /// The function returns true if a splitting occurred within t <=
  /// tmax, otherwise t is set to tmax and the function returns false.
  /// 
  bool sudakov_split_quark(int index, double & t, const double tmax, const double epsilon) {
    double x1 = epsilon / _events.back()[index].x();
    double x2 = 1 - x1;
    // there is no splitting probability in this case
    if (x2 < x1) {
      t = tmax;
      return false;
    }

    double pqq_int = Pqq_sup_int(x1, x2);
    double mu = 1.0 / Pqq_sup_int(x1, x2);
    
    while (true) {
      t += _rng.exponential(mu);
      // never go beyond tmax
      if (t > tmax) {
        t = tmax;
        return false;
      } 

      // now do the splitting; 
      // solve r = log((1-x)/(1-x2)) / log((1-x1)/(1-x2)) -> 
      //   ->  log((1-x)/(1-x2)) = r * log((1-x1)/(1-x2))
      //   ->      (1-x)         = (1-x2)*exp(r * log((1-x1)/(1-x2)) )
      //   ->       x = 1 - (1-x2)*exp(r * log((1-x1)/(1-x2)) )

      // solve r = 2CF log((1-x)/(1-x2)) / pqq_int -> 
      //   ->  2CF log((1-x)/(1-x2)) = r * pqq_int
      //   ->      (1-x)         = (1-x2)*exp(r * log((1-x1)/(1-x2)) /(2CF) )
      //   ->       x = 1 - (1-x2)*exp(r * log((1-x1)/(1-x2)) /(2CF))
      double z = 1 - (1-x2)*exp( _rng.uniform() * pqq_int/(2*_CF) );
      double accept_prob = Pqq(z)/Pqq_sup(z);
      if (_rng.uniform() < accept_prob) {
        split(index, z, Parton::q, Parton::g);
        return true;
      }
    }
  }

  /// take the gluon with the specified index and specified t value,
  /// and split it, updating the t value, and adding the new emission
  /// to the end of our array. The divergences near x=1 (and if
  /// relevant near x=0) are regularised by cutoff epsilon.
  ///
  /// The function returns true if a splitting occurred within t <=
  /// tmax, otherwise t is set to tmax and the function returns false.
  /// 
  bool sudakov_split_gluon(int index, double & t, const double tmax, const double epsilon) {
    
    double x1 = epsilon / _events.back()[index].x();
    double x2 = 1 - x1;
    // there is no splitting probability in this case
    if (x2 < x1) {
      t = tmax;
      return false;
    }
    double pgg_int = Pgg_asym_sup_int(x1, x2);
    double pqg_int = Pqg_sup_int(x1, x2);
    double mu = 1.0 / (pgg_int + pqg_int);

    while (true) {
      t += _rng.exponential(mu);
      // never go beyond tmax
      if (t > tmax) {
        t = tmax;
        return false;
      } 

      double z, accept_prob;
      // now decide between quark and gluon
      if (_rng.uniform() < pgg_int*mu) {
        // (gg) <- g; see the quark splitting above for the derivation of the formula
        z = 1 - (1-x2)*exp( _rng.uniform() * pgg_int/(2*_CA) );
        accept_prob = Pgg_asym(z)/Pgg_asym_sup(z);
        if (_rng.uniform() < accept_prob) {
          split(index, z, Parton::g, Parton::g);
          return true;
        }
      } else {
        // (qq) <- g
        z = _rng.uniform(x1, x2);
        accept_prob = Pqg(z)/Pqg_sup(z);
        assert(accept_prob < 1 + 1e-6);
        if (_rng.uniform() < accept_prob) {
          split(index, z, Parton::q, Parton::q);
          return true;
        }
      }
      // safety test, where 1e-6 is a tolerance for rounding errors
    }
  }

  /// calls sudakov_split_gluon or sudakov_split_quark depending on the flavour
  /// of the parton with the given index
  bool sudakov_split(int index, double & t, const double tmax, const double epsilon) {
    if (_events.back()[index].flav() == Parton::g) {
      return sudakov_split_gluon(index, t, tmax, epsilon);
    } else if (_events.back()[index].flav() == Parton::q) {
      return sudakov_split_quark(index, t, tmax, epsilon);
    } else {
      std::cerr << "ERROR in sudakov_split: unrecognised flavour: " 
                << _events.back()[index].flav()
                << std::endl;
      std::exit(-1);
    }
  }

  /// carry out recursive splitting in order to obtain a full configuration
  void sudakov_recursive_split(const int index, double t, 
                               const double tmax, const double epsilon) {
    int    new_index = _events.back().size();
    // try to split a configuration. If successful, the new particles
    // will have "newt" and will be at indices index and
    // new_index. Then recursively split those particles too.
    if (sudakov_split(index, t, tmax, epsilon)) {
      sudakov_recursive_split(index,     t, tmax, epsilon);
      sudakov_recursive_split(new_index, t, tmax, epsilon);
    }
  }
};

#endif // __ALLORDERGENERATORRECURSIVE_HH__
