!----------------------------------------------------------------------
!! A module that helps with the "setup" tasks that are potentially
!! common to multiple programs
module common_setup
  use hoppet_v1, EvolvePDF_hoppet => EvolvePDF, InitPDF_hoppet => InitPDF
  use sub_defs_io ! to get access to the command-line
  implicit none

  !! default hoppet parameters
  real(dp) :: dy = 0.02_dp, du = 0.025_dp, ymax = 20.0_dp
  integer  :: order = -6

  !! core hoppet objects
  type(grid_def), save :: grid
  ! holds the splitting functions
  type(dglap_holder), save :: dh

  !! quantities realated to the t <-> hoppet scale conversion
  real(dp) :: hoppet_Q0 = 1.0_dp, hoppet_asMZ = 0.118_dp
  integer  :: hoppet_nf = 5
  integer  :: hoppet_idev
  type(running_coupling), save :: hoppet_coupling

  real(dp) :: hoppet_asQ0, b0
  
contains

  !----------------------------------------------------------------------
  subroutine do_common_setup()

    ! set up parameters for grid
    order = int_val_opt("-order", order)
    ymax  = dble_val_opt("-ymax", ymax)
    dy    = dble_val_opt('-dy',dy)
    du    = dble_val_opt('-du',du)
    hoppet_nf   = int_val_opt ("-hoppet-nf",   hoppet_nf)

    ! set up hoppet
    call InitGridDef(grid, dy, ymax, order=order)
    call SetDefaultEvolutionDu(du)
    call InitDglapHolder(grid,dh,factscheme=factscheme_MSbar,&
         &                      nloop=1, nflo=hoppet_nf, nfhi=hoppet_nf)

    ! setup up a toy running coupling (needed for internal evolution
    ! and conversion between t & hoppet_Q)
    hoppet_Q0   = dble_val_opt("-hoppet-Q0",   hoppet_Q0)
    hoppet_asmZ = dble_val_opt("-hoppet-asMZ", hoppet_asMZ)

    call InitRunningCoupling(hoppet_coupling,&
         &                   alfas=hoppet_asMZ,Q=91.2_dp,nloop=1, fixnf=hoppet_nf)

    ! store the info we need
    b0 = beta0 * twopi
    hoppet_asQ0 = Value(hoppet_coupling,hoppet_Q0)

  end subroutine do_common_setup

  !----------------------------------------------------------------------
  !! handle deletion of left-over objects
  subroutine common_delete()
    call Delete(dh)
    call Delete(hoppet_coupling)
    call Delete(grid)
  end subroutine common_delete
  
  
  !----------------------------------------------------------------------
  subroutine print_common_setup(idev)
    integer, intent(in) :: idev

    hoppet_idev = idev
    write(idev,'(a)') '# '//trim(command_line())
    write(idev,'(a,f8.5)') '# dy = ',dy
    write(idev,'(a,f8.3)') '# ymax = ',ymax
    write(idev,'(a,f8.5)') '# du = ',du
    write(idev,'(a,i5)')   '# order = ',order

    write(idev,'(a,f8.5)') '# hoppet_asMZ = ',hoppet_asMZ
    write(idev,'(a,f8.5)') '# hoppet_Q0   = ',hoppet_Q0
    write(idev,'(a,f8.5)') '# hoppet_asQ0 = ',hoppet_asQ0

  end subroutine print_common_setup
  
  !----------------------------------------------------------------------
  ! t with upwards Q evolution is log(1 + b0 alpha(Q0)/twopi * log(Q^2/Q0^2))/(b0)
  real(dp) function t_of_hoppetQ(Q_in)
    real(dp), intent(in) :: Q_in
    t_of_hoppetQ = log(1 + b0*log(Q_in**2/hoppet_Q0**2)*hoppet_asQ0/twopi)/b0
  end function t_of_hoppetQ

  !----------------------------------------------------------------------
  !    exp(b0 t) = 1 + b0 alpha(Q0)/twopi ln (Q^2/Q0^2)
  ! -> 
  real(dp) function hoppetQ_of_t(t_in)
    real(dp), intent(in) :: t_in
    hoppetQ_of_t = hoppet_Q0 * exp(half * (exp(b0*t_in) - one)/(b0*hoppet_asQ0/twopi))
  end function hoppetQ_of_t

  !----------------------------------------------------------------------
  ! works out \int_0^ymax dy exp(-y * moment_index) * pdf
  function dumb_moment(pdf_in, moment_index)
    real(dp), intent(in)  :: pdf_in(0:)
    real(dp), intent(in)  :: moment_index
    real(dp)              :: dumb_moment
    !----------------------------------

    dumb_moment = grid%dy*sum(exp(-yValues(grid)*moment_index)*pdf_in)
  end function dumb_moment

end module common_setup
