!----------------------------------------------------------------------
!! A module to compute the hoppet evolution of a given partonic spectrum

module hoppet_convolutions
  use common_setup
  use hoppet_v1, EvolvePDF_hoppet => EvolvePDF, InitPDF_hoppet => InitPDF
  use sub_defs_io ! to get access to the command-line
  implicit none  
  integer, parameter :: ntexp=3 ! maximum number of powers in t expansion
  ! microjet_spectrum is defined by the following:
  !  - first index is pt index,
  !  - second is for initial gluon (0) or quark (1) 
  !  - third index gluon microjets (0) or quark microjets (1)
  real(dp), pointer :: microjet_spectrum(:,:,:)
  ! microjets_as and microjets_t are defined by the following:
  !  - first index (0, grid%ny) is pt index,
  !  - second index (0,1) is for evolution, starting from an initial
  !    gluon (0) or quark (1), 
  !  - third index (0,ntexp) is order in expansion in alphas or t
  real(dp), pointer :: microjets_as(:,:,:), microjets_t(:,:,:)
  real(dp), pointer :: from_quark(:,:), from_quark_texp(:,:,:)
  real(dp), pointer :: from_gluon(:,:), from_gluon_texp(:,:,:)
  real(dp) :: R, R0, sqrts
  ! If not using LHAPDF, use toy model for pdf coupling
  type(running_coupling) :: pdf_coupling

 

contains
  
  ! Do hoppet convolution of parton spectrum
  ! and return resummed spectrum as well as expansion in t and alphas
  subroutine hoppet_conv(pts, rts, parton_spectrum, xmur, spectrum_type)
    real(dp), pointer    :: pts(:), parton_spectrum(:,:)
    real(dp), intent(in) :: rts, xmur
    integer , intent(in) :: spectrum_type
    logical  :: use_lhapdf
    logical           :: scale_ptoverz
    real(dp) :: hoppetQ, last_hoppetQ, alphas_pt, t_pt
    integer :: ipt
    !real(dp), parameter :: MZ=91.2_dp
    real(dp), external  :: alphasPDF
    
    use_lhapdf = (spectrum_type.eq.2).or.(spectrum_type.eq.0)
    sqrts = rts
    R0 = dble_val_opt("-R0", 1.0_dp)
    R  = dble_val_opt("-R" , 0.4_dp)

    write(hoppet_idev,'(a,f4.2)') '# R = ', R
    write(hoppet_idev,'(a,f4.2)') '# R0 = ', R0

    ! use t(R,pt) or t(R,pt/z) (default is pt/z)
    scale_ptoverz = .not.log_val_opt("-scale-pt")
    
    ! set up the alphas structure -- for now set up a toy 3-loop nf=5
    ! coupling from hoppet just to mimic what we would get from an
    ! actual PDF
    if (.not.use_lhapdf) then
       call InitRunningCoupling(pdf_coupling,&
            &                   alfas=0.118_dp,Q=91.2_dp,nloop=3, fixnf=5)
    endif

    ! initialise our "transfer" functions
    call setup_flavour(from_gluon, from_gluon_texp, 0, ntexp)
    call setup_flavour(from_quark, from_quark_texp, 1, ntexp)

    ! now try to get the jet spectrum
    last_hoppetQ = hoppet_Q0
    call AllocGridQuant(grid, microjet_spectrum,0,1,0,1)
    microjet_spectrum = zero
    call AllocGridQuant(grid, microjets_as,0,1,0,ntexp)
    microjets_as = zero
    call AllocGridQuant(grid, microjets_t,0,1,0,ntexp)
    microjets_t = zero

    !----------------------------------------------------------------------
    ! Calculate the jet specrum after small-R resummation, as well as
    ! fixed order expansion in t and alphas
    ! 
    do ipt = 0, grid%ny

       if (use_lhapdf) then
          alphas_pt = alphasPDF(xmur*pts(ipt))
       else
          alphas_pt = Value(pdf_coupling, xmur*pts(ipt))
       endif

       t_pt = t_of_alphaspt_R(alphas_pt, R/R0)
       
       ! now do a little bit of evolution
       hoppetQ = hoppetQ_of_t(t_pt)
       !write(0,*) t_pt, hoppetQ, last_hoppetQ
       call EvolvePDF_hoppet(dh, from_quark, hoppet_coupling, last_hoppetQ, hoppetQ)
       call EvolvePDF_hoppet(dh, from_gluon, hoppet_coupling, last_hoppetQ, hoppetQ)
       last_hoppetQ = hoppetQ


       !----------------------------------------------------------------------
       ! First we implement the scheme where we have t(pt/z), i.e. scale is chosen
       ! to be that of primary parton pt
       !
       if (scale_ptoverz) then
          ! all-order gluon/quark microjet spectrum
          call resum_spectrum(ipt, grid%ny, grid%dy, from_gluon, from_quark, &
               & parton_spectrum, microjet_spectrum)

          ! now get t expansion of microjet spectrum
          call t_spectrum(ipt, grid%ny, grid%dy, t_pt, ntexp, &
               & from_gluon_texp, from_quark_texp, parton_spectrum, microjets_t)

          ! now get alphas expansion of microjet spectrum
          call as_spectrum(ipt, grid%ny, grid%dy, alphas_pt*log(R/R0), ntexp, &
               & from_gluon_texp, from_quark_texp, parton_spectrum, microjets_as)

       !----------------------------------------------------------------------
       ! Now we implement the scheme where we have t(pt), i.e. scale is chosen
       ! to be that of microjet pt
       !
       else 
          ! all-order gluon/quark microjet spectrum
          call resum_spectrum_scalept(ipt, grid%dy, from_gluon, from_quark, &
               & parton_spectrum, microjet_spectrum)

          ! now get t expansion of microjet spectrum
          call t_spectrum_scalept(ipt, grid%dy, t_pt, ntexp, &
               & from_gluon_texp, from_quark_texp, parton_spectrum, microjets_t)

          ! now get alphas expansion of microjet spectrum
          call as_spectrum_scalept(ipt, grid%dy, alphas_pt*log(R/R0), ntexp, &
               & from_gluon_texp, from_quark_texp, parton_spectrum, microjets_as)
       endif
    end do

  end subroutine hoppet_conv

  subroutine resum_spectrum (ipt, ny, dy, from_gluon, from_quark, &
       & parton_spectrum, microjet_spectrum)
    integer, intent(in) :: ipt, ny
    real(dp), intent(in) :: dy
    real(dp), pointer :: from_gluon(:,:), from_quark(:,:), parton_spectrum(:,:)
    real(dp), pointer :: microjet_spectrum(:,:,:)

    ! all-order gluon microjet spectrum, from an initial gluon
    microjet_spectrum(ipt:,0,0) = microjet_spectrum(ipt:,0,0) &
         & + dy * parton_spectrum(ipt,0) * from_gluon(0:ny-ipt,0)

    ! all-order quark microjet spectrum, from an initial gluon
    microjet_spectrum(ipt:,0,1) = microjet_spectrum(ipt:,0,1) &
         & + dy * parton_spectrum(ipt,0)*(sum(from_gluon(0:ny-ipt,-6:-1),dim=2) &
         &                                + sum(from_gluon(0:ny-ipt,1:6),dim=2))
    
    ! all-order gluon microjet spectrum, from an initial quark
    microjet_spectrum(ipt:,1,0) = microjet_spectrum(ipt:,1,0) &
         & + dy * parton_spectrum(ipt,1) * from_quark(0:ny-ipt,0)

    ! all-order quark microjet spectrum, from an initial quark
    microjet_spectrum(ipt:,1,1) = microjet_spectrum(ipt:,1,1) &
         & + dy * parton_spectrum(ipt,1)*(sum(from_quark(0:ny-ipt,-6:-1),dim=2) &
         &                                + sum(from_quark(0:ny-ipt,1:6),dim=2))

  end subroutine resum_spectrum

  subroutine resum_spectrum_scalept (ipt, dy, from_gluon, from_quark, &
       & parton_spectrum, microjet_spectrum)
    integer, intent(in) :: ipt
    real(dp), intent(in) :: dy
    real(dp), pointer :: from_gluon(:,:), from_quark(:,:), parton_spectrum(:,:)
    real(dp), pointer :: microjet_spectrum(:,:,:)

    ! all-order gluon microjet spectrum from an initial gluon
    microjet_spectrum(ipt,0,0) = dy * sum(parton_spectrum(ipt:0:-1,0) * &
         &                                from_gluon(0:ipt,0))

    ! all-order quark microjet spectrum from an initial gluon
    microjet_spectrum(ipt,0,1) = dy * sum(parton_spectrum(ipt:0:-1,0) * &
         & (sum(from_gluon(0:ipt,-6:-1),dim=2) + sum(from_gluon(0:ipt,1:6),dim=2)))

    ! all-order gluon microjet spectrum from an initial quark
    microjet_spectrum(ipt,1,0) = dy * sum(parton_spectrum(ipt:0:-1,1) * &
         &                                from_quark(0:ipt,0))

    ! all-order quark microjet spectrum from an initial quark
    microjet_spectrum(ipt,1,1) = dy * sum(parton_spectrum(ipt:0:-1,1) * &
         & (sum(from_quark(0:ipt,-6:-1),dim=2) + sum(from_quark(0:ipt,1:6),dim=2)))
    
  end subroutine resum_spectrum_scalept

  subroutine t_spectrum (ipt, ny, dy, t_pt, ntexp, from_gluon_texp, from_quark_texp, &
       & parton_spectrum, microjets_t)
    integer, intent(in) :: ipt, ny, ntexp
    real(dp), intent(in) :: dy, t_pt
    real(dp), pointer :: from_gluon_texp(:,:,:), from_quark_texp(:,:,:)
    real(dp), pointer :: parton_spectrum(:,:)
    real(dp), pointer :: microjets_t(:,:,:)
    integer :: itexp
    real(dp) :: tpow
    !! t expansion
    tpow = one
    do itexp=0,ntexp
       if (itexp.gt.0) then
          tpow = tpow * t_pt/itexp
       endif
       ! t expansion of initial gluon to quark/gluon microjets
       microjets_t(ipt:,0,itexp) = microjets_t(ipt:,0,itexp) &
            &         + tpow * dy * parton_spectrum(ipt,0) * &
            &           sum(from_gluon_texp(0:ny-ipt,-6:6,itexp),dim=2)
       ! t expansion of intial quark to quark/gluon microjets
       microjets_t(ipt:,1,itexp) = microjets_t(ipt:,1,itexp) &
            &         + tpow * dy * parton_spectrum(ipt,1) * &
            &           sum(from_quark_texp(0:ny-ipt,-6:6,itexp),dim=2)
    end do
    
  end subroutine t_spectrum

  subroutine t_spectrum_scalept (ipt, dy, t_pt, ntexp, from_gluon_texp, &
       & from_quark_texp, parton_spectrum, microjets_t)
    integer, intent(in) :: ipt, ntexp
    real(dp), intent(in) :: dy, t_pt
    real(dp), pointer :: from_gluon_texp(:,:,:), from_quark_texp(:,:,:)
    real(dp), pointer :: parton_spectrum(:,:)
    real(dp), pointer :: microjets_t(:,:,:)
    integer :: itexp
    real(dp) :: tpow
    !! t expansion    
    tpow = one
    do itexp=0,ntexp
       if (itexp.gt.0) then
          tpow = tpow * t_pt/itexp
       endif
       ! t expansion of initial gluon to quark/gluon microjets
       microjets_t(ipt,0,itexp) = dy * tpow * &
            &               sum(parton_spectrum(ipt:0:-1,0) * &
            &               sum(from_gluon_texp(0:ipt,-6:6,itexp),dim=2))
       ! t expansion of initial quark to quark/gluon microjets
       microjets_t(ipt,1,itexp) = dy * tpow * &
            &               sum(parton_spectrum(ipt:0:-1,1) * &
            &               sum(from_quark_texp(0:ipt,-6:6,itexp),dim=2))
    end do
  end subroutine t_spectrum_scalept

  subroutine as_spectrum (ipt, ny, dy, aslnR, ntexp, from_gluon_texp, &
       & from_quark_texp, parton_spectrum, microjets_as)
    integer, intent(in) :: ipt, ny, ntexp
    real(dp), intent(in) :: dy, aslnR
    real(dp), pointer :: from_gluon_texp(:,:,:), from_quark_texp(:,:,:)
    real(dp), pointer :: parton_spectrum(:,:)
    real(dp), pointer :: microjets_as(:,:,:)

    !! expansion in alphas
    ! t c1 + t^2 c2/2 + t^3 c3/6
    !  =
    !  - ( (c1 log[R] as) / Pi)
    !  + ( (b0 c1 + c2) log[R]^2 as^2 ) / (2 Pi^2)
    !  - ( (2 b0^2 c1 + 3 b0 c2 + c3) log[R]^3 as^3 ) / (6 Pi^3)
    !  + O[as]^4
    
    ! O(1)
    microjets_as(ipt:,0,0) = microjets_as(ipt:,0,0) &
         &            + dy * parton_spectrum(ipt,0) * &
         &              sum(from_gluon_texp(0:ny-ipt,-6:6,0),dim=2)
    microjets_as(ipt:,1,0) = microjets_as(ipt:,1,0) &
         &            + dy * parton_spectrum(ipt,1) * &
         &              sum(from_quark_texp(0:ny-ipt,-6:6,0),dim=2)

    ! O(alphas)
    if (ntexp.gt.0) then
       microjets_as(ipt:,0,1) = microjets_as(ipt:,0,1) + &
            & (-aslnR/pi)*dy * parton_spectrum(ipt,0) * &
            &              sum(from_gluon_texp(0:ny-ipt,-6:6,1),dim=2)
       microjets_as(ipt:,1,1) = microjets_as(ipt:,1,1) + &
            & (-aslnR/pi)*dy * parton_spectrum(ipt,1) * &
            &              sum(from_quark_texp(0:ny-ipt,-6:6,1),dim=2)
    endif
    
    ! O(alphas^2) 
    if (ntexp.gt.1) then
       microjets_as(ipt:,0,2) = microjets_as(ipt:,0,2) + &
            & (aslnR)**2/(two*(pi**2)) * (&
            & b0*dy * parton_spectrum(ipt,0) * &
            &               sum(from_gluon_texp(0:ny-ipt,-6:6,1),dim=2)&
            & + dy * parton_spectrum(ipt,0) * &
            &               sum(from_gluon_texp(0:ny-ipt,-6:6,2),dim=2))
       microjets_as(ipt:,1,2) = microjets_as(ipt:,1,2) + &
            & (aslnR)**2/(two*(pi**2)) * (&
            & b0*dy * parton_spectrum(ipt,1) * &
            &              sum(from_quark_texp(0:ny-ipt,-6:6,1),dim=2)&
            & + dy * parton_spectrum(ipt,1) * &
            &              sum(from_quark_texp(0:ny-ipt,-6:6,2),dim=2))
    endif
    ! O(alphas^3)
    if (ntexp.gt.2) then
       microjets_as(ipt:,0,3) = microjets_as(ipt:,0,3) - & 
            & aslnR**3/ (6_dp*(pi**3)) * (&
            & 2*b0*b0*dy * parton_spectrum(ipt,0) * &
            &               sum(from_gluon_texp(0:ny-ipt,-6:6,1),dim=2)&
            & + 3*b0*dy * parton_spectrum(ipt,0) * &
            &               sum(from_gluon_texp(0:ny-ipt,-6:6,2),dim=2)&
            & + dy * parton_spectrum(ipt,0) * &
            &               sum(from_gluon_texp(0:ny-ipt,-6:6,3),dim=2))
       microjets_as(ipt:,1,3) = microjets_as(ipt:,1,3) - & 
            & aslnR**3/ (6_dp*(pi**3)) * (&
            & 2*b0*b0*dy * parton_spectrum(ipt,1) * &
            &               sum(from_quark_texp(0:ny-ipt,-6:6,1),dim=2)&
            & + 3*b0*dy * parton_spectrum(ipt,1) * &
            &               sum(from_quark_texp(0:ny-ipt,-6:6,2),dim=2)&
            & + dy * parton_spectrum(ipt,1) * &
            &               sum(from_quark_texp(0:ny-ipt,-6:6,3),dim=2))
    endif
    
  end subroutine as_spectrum

  subroutine as_spectrum_scalept (ipt, dy, aslnR, ntexp, from_gluon_texp, &
       & from_quark_texp, parton_spectrum, microjets_as)
    integer, intent(in) :: ipt, ntexp
    real(dp), intent(in) :: dy, aslnR
    real(dp), pointer :: from_gluon_texp(:,:,:), from_quark_texp(:,:,:)
    real(dp), pointer :: parton_spectrum(:,:)
    real(dp), pointer :: microjets_as(:,:,:)

        !! expansion in alphas
        ! t c1 + t^2 c2/2 + t^3 c3/6
        !  =
        !  - ( (c1 log[R] as) / Pi)
        !  + ( (b0 c1 + c2) log[R]^2 as^2 ) / (2 Pi^2)
        !  - ( (2 b0^2 c1 + 3 b0 c2 + c3) log[R]^3 as^3 ) / (6 Pi^3)
        !  + O[as]^4
        
        ! O(1)
        microjets_as(ipt,0,0) = dy * sum(parton_spectrum(ipt:0:-1,0) *&
             &              sum(from_gluon_texp(0:ipt,-6:6,0),dim=2))
        microjets_as(ipt,1,0) = dy * sum(parton_spectrum(ipt:0:-1,1) *&
             &              sum(from_quark_texp(0:ipt,-6:6,0),dim=2))
        
        ! O(alphas)
        if (ntexp.gt.0) then
           microjets_as(ipt,0,1) = (-aslnR/pi) * dy * &
                &              sum(parton_spectrum(ipt:0:-1,0) * &
                &              sum(from_gluon_texp(0:ipt,-6:6,1),dim=2))
           microjets_as(ipt,1,1) = (-aslnR/pi) *dy * &
                &              sum(parton_spectrum(ipt:0:-1,1) * &
                &              sum(from_quark_texp(0:ipt,-6:6,1),dim=2))
        endif

        ! O(alphas^2) 
        if (ntexp.gt.1) then
           microjets_as(ipt,0,2) = (aslnR**2/(two*(pi**2))) * (&
                & b0*dy*sum(parton_spectrum(ipt:0:-1,0)* &
                &           sum(from_gluon_texp(0:ipt,-6:6,1),dim=2)) &
                & + dy * sum(parton_spectrum(ipt:0:-1,0) *&
                &           sum(from_gluon_texp(0:ipt,-6:6,2),dim=2)))
           microjets_as(ipt,1,2) = (aslnR**2/(two*(pi**2))) * (&
                & b0*dy*sum(parton_spectrum(ipt:0:-1,1)* &
                &           sum(from_quark_texp(0:ipt,-6:6,1),dim=2))&
                & + dy * sum(parton_spectrum(ipt:0:-1,1) *&
                &           sum(from_quark_texp(0:ipt,-6:6,2),dim=2)))
        endif
     
        ! O(alphas^3)
        if (ntexp.gt.2) then
           microjets_as(ipt,0,3) = (-aslnR**3/ (6_dp*(pi**3))) * (&
             & 2*b0*b0*dy * sum(parton_spectrum(ipt:0:-1,0) *&
             &              sum(from_gluon_texp(0:ipt,-6:6,1),dim=2))&
             & + 3*b0*dy * sum(parton_spectrum(ipt:0:-1,0) *&
             &              sum(from_gluon_texp(0:ipt,-6:6,2),dim=2))&
             & + dy * sum(parton_spectrum(ipt:0:-1,0) *&
             &              sum(from_gluon_texp(0:ipt,-6:6,3),dim=2)))
           microjets_as(ipt,1,3) = (-aslnR**3/ (6_dp*(pi**3))) * (&
             & 2*b0*b0*dy * sum(parton_spectrum(ipt:0:-1,1) *&
             &              sum(from_quark_texp(0:ipt,-6:6,1),dim=2))&
             & + 3*b0*dy * sum(parton_spectrum(ipt:0:-1,1) *&
             &              sum(from_quark_texp(0:ipt,-6:6,2),dim=2))&
             & + dy * sum(parton_spectrum(ipt:0:-1,1) *&
             &              sum(from_quark_texp(0:ipt,-6:6,3),dim=2)))
        endif

  end subroutine as_spectrum_scalept

  
  !----------------------------------------------------------------------
  subroutine setup_flavour(from_flav, from_flav_texp, flav_index, ntexp)
    real(dp), pointer    :: from_flav(:,:), from_flav_texp(:,:,:)
    integer,  intent(in) :: flav_index, ntexp
    integer :: it

    call AllocPDF(grid, from_flav)
    call AllocPDF(grid, from_flav_texp, 0, ntexp)
    from_flav               = 0
    from_flav(0,flav_index) = one/grid%dy
    
    ! get the t expansion
    from_flav_texp(:,:,0) = from_flav
    do it = 1, ntexp
      from_flav_texp(:,:,it) = dh%P_LO * from_flav_texp(:,:,it-1)
    end do

  end subroutine setup_flavour
  !----------------------------------------------------------------------
  real(dp) function t_of_alphaspt_R(alphas_pt_in, R_in)
    real(dp), intent(in) :: alphas_pt_in, R_in
    t_of_alphaspt_R = -log(1 + b0*log(R_in)*alphas_pt_in/pi)/b0
  end function t_of_alphaspt_R

  ! continuous function of pt returning the resummed gluon microjet spectrum
  ! from the initial gluon spectrum
  real(dp) function gluon_microjet_spec_from_gluon (pt)
    real(dp) :: y
    real(dp), intent(in) :: pt
    y = log(sqrts/pt)
    gluon_microjet_spec_from_gluon = EvalGridQuant(grid,microjet_spectrum(:,0,0), y)
  end function gluon_microjet_spec_from_gluon

  ! continuous function of pt returning the resummed quark microjet spectrum
  ! from the initial gluon spectrum
  real(dp) function quark_microjet_spec_from_gluon (pt)
    real(dp) :: y
    real(dp), intent(in) :: pt
    y = log(sqrts/pt)
    quark_microjet_spec_from_gluon = EvalGridQuant(grid,microjet_spectrum(:,0,1), y)
  end function quark_microjet_spec_from_gluon

  ! continuous function of pt returning the resummed microjet spectrum
  ! from initial gluons
  real(dp) function microjet_spec_from_gluon (pt)
    real(dp) :: y
    real(dp), intent(in) :: pt
    y = log(sqrts/pt)
    microjet_spec_from_gluon = EvalGridQuant(grid,sum(microjet_spectrum(:,0,:),dim=2), y)
  end function microjet_spec_from_gluon
 
  ! continuous function of pt returning the resummed gluon microjet spectrum
  ! from the initial quark spectrum
  real(dp) function gluon_microjet_spec_from_quark (pt)
    real(dp) :: y
    real(dp), intent(in) :: pt
    y = log(sqrts/pt)
    gluon_microjet_spec_from_quark = EvalGridQuant(grid,microjet_spectrum(:,1,0), y)
  end function gluon_microjet_spec_from_quark

  ! continuous function of pt returning the resummed quark microjet spectrum
  ! from the initial quark spectrum
  real(dp) function quark_microjet_spec_from_quark (pt)
    real(dp) :: y
    real(dp), intent(in) :: pt
    y = log(sqrts/pt)
    quark_microjet_spec_from_quark = EvalGridQuant(grid,microjet_spectrum(:,1,1), y)
  end function quark_microjet_spec_from_quark

  ! continuous function of pt returning the resummed microjet spectrum
  ! from initial quarks
  real(dp) function microjet_spec_from_quark (pt)
    real(dp) :: y
    real(dp), intent(in) :: pt
    y = log(sqrts/pt)
    microjet_spec_from_quark = EvalGridQuant(grid,sum(microjet_spectrum(:,1,:),dim=2), y)
  end function microjet_spec_from_quark

  ! returns the t^ord contribution to the gluon microjet spectrum
  real(dp) function microjets_g_t0 (pt)
    real(dp) :: y
    integer  :: ord
    real(dp), intent(in) :: pt
    y = log(sqrts/pt)
    ord = min(ntexp,0)
    microjets_g_t0 = EvalGridQuant(grid,microjets_t(:,0,ord), y)
  end function microjets_g_t0
  real(dp) function microjets_g_t1 (pt)
    real(dp) :: y
    integer  :: ord
    real(dp), intent(in) :: pt
    y = log(sqrts/pt)
    ord = min(ntexp,1)
    microjets_g_t1 = EvalGridQuant(grid,microjets_t(:,0,ord), y)
  end function microjets_g_t1
  real(dp) function microjets_g_t2 (pt)
    real(dp) :: y
    integer  :: ord
    real(dp), intent(in) :: pt
    y = log(sqrts/pt)
    ord = min(ntexp,2)
    microjets_g_t2 = EvalGridQuant(grid,microjets_t(:,0,ord), y)
  end function microjets_g_t2
  real(dp) function microjets_g_t3 (pt)
    real(dp) :: y
    integer  :: ord
    real(dp), intent(in) :: pt
    y = log(sqrts/pt)
    ord = min(ntexp,3)
    microjets_g_t3 = EvalGridQuant(grid,microjets_t(:,0,ord), y)
  end function microjets_g_t3
  
  ! returns the t^ord contribution to the quark microjet spectrum
  real(dp) function microjets_q_t0 (pt)
    real(dp) :: y
    integer  :: ord
    real(dp), intent(in) :: pt
    y = log(sqrts/pt)
    ord = min(ntexp,0)
    microjets_q_t0 = EvalGridQuant(grid,microjets_t(:,1,ord), y)
  end function microjets_q_t0
  real(dp) function microjets_q_t1 (pt)
    real(dp) :: y
    integer  :: ord
    real(dp), intent(in) :: pt
    y = log(sqrts/pt)
    ord = min(ntexp,1)
    microjets_q_t1 = EvalGridQuant(grid,microjets_t(:,1,ord), y)
  end function microjets_q_t1
  real(dp) function microjets_q_t2 (pt)
    real(dp) :: y
    integer  :: ord
    real(dp), intent(in) :: pt
    y = log(sqrts/pt)
    ord = min(ntexp,2)
    microjets_q_t2 = EvalGridQuant(grid,microjets_t(:,1,ord), y)
  end function microjets_q_t2
  real(dp) function microjets_q_t3 (pt)
    real(dp) :: y
    integer  :: ord
    real(dp), intent(in) :: pt
    y = log(sqrts/pt)
    ord = min(ntexp,3)
    microjets_q_t3 = EvalGridQuant(grid,microjets_t(:,1,ord), y)
  end function microjets_q_t3

  ! returns the t^ord contribution to the quark+gluon microjet spectrum
  real(dp) function microjets_sum_t0 (pt)
    real(dp), intent(in) :: pt
    microjets_sum_t0 =  microjets_g_t0(pt) + microjets_q_t0(pt) 
  end function microjets_sum_t0
  real(dp) function microjets_sum_t1 (pt)
    real(dp), intent(in) :: pt
    microjets_sum_t1 = microjets_g_t1(pt) + microjets_q_t1(pt)    
  end function microjets_sum_t1
  real(dp) function microjets_sum_t2 (pt)
    real(dp), intent(in) :: pt
    microjets_sum_t2 = microjets_g_t2(pt) + microjets_q_t2(pt)    
  end function microjets_sum_t2
  real(dp) function microjets_sum_t3 (pt)
    real(dp), intent(in) :: pt
    microjets_sum_t3 = microjets_g_t3(pt) + microjets_q_t3(pt)    
  end function microjets_sum_t3

  ! returns the alphas^ord contribution to the gluon microjet spectrum
  real(dp) function microjets_g_as0 (pt)
    real(dp) :: y
    integer  :: ord
    real(dp), intent(in) :: pt
    y = log(sqrts/pt)
    ord = min(ntexp,0)
    microjets_g_as0 = EvalGridQuant(grid,microjets_as(:,0,ord), y)
  end function microjets_g_as0
  real(dp) function microjets_g_as1 (pt)
    real(dp) :: y
    integer  :: ord
    real(dp), intent(in) :: pt
    y = log(sqrts/pt)
    ord = min(ntexp,1)
    microjets_g_as1 = EvalGridQuant(grid,microjets_as(:,0,ord), y)
  end function microjets_g_as1
  real(dp) function microjets_g_as2 (pt)
    real(dp) :: y
    integer  :: ord
    real(dp), intent(in) :: pt
    y = log(sqrts/pt)
    ord = min(ntexp,2)
    microjets_g_as2 = EvalGridQuant(grid,microjets_as(:,0,ord), y)
  end function microjets_g_as2
  real(dp) function microjets_g_as3 (pt)
    real(dp) :: y
    integer  :: ord
    real(dp), intent(in) :: pt
    y = log(sqrts/pt)
    ord = min(ntexp,3)
    microjets_g_as3 = EvalGridQuant(grid,microjets_as(:,0,ord), y)
  end function microjets_g_as3
  
  ! returns the alphas^ord contribution to the quark microjet spectrum
  real(dp) function microjets_q_as0 (pt)
    real(dp) :: y
    integer  :: ord
    real(dp), intent(in) :: pt
    y = log(sqrts/pt)
    ord = min(ntexp,0)
    microjets_q_as0 = EvalGridQuant(grid,microjets_as(:,1,ord), y)    
  end function microjets_q_as0
  real(dp) function microjets_q_as1 (pt)
    real(dp) :: y
    integer  :: ord
    real(dp), intent(in) :: pt
    y = log(sqrts/pt)
    ord = min(ntexp,1)
    microjets_q_as1 = EvalGridQuant(grid,microjets_as(:,1,ord), y)    
  end function microjets_q_as1
  real(dp) function microjets_q_as2 (pt)
    real(dp) :: y
    integer  :: ord
    real(dp), intent(in) :: pt
    y = log(sqrts/pt)
    ord = min(ntexp,2)
    microjets_q_as2 = EvalGridQuant(grid,microjets_as(:,1,ord), y)    
  end function microjets_q_as2
  real(dp) function microjets_q_as3 (pt)
    real(dp) :: y
    integer  :: ord
    real(dp), intent(in) :: pt
    y = log(sqrts/pt)
    ord = min(ntexp,3)
    microjets_q_as3 = EvalGridQuant(grid,microjets_as(:,1,ord), y)    
  end function microjets_q_as3
 
  ! returns the alphas^ord contribution to the quark+gluon microjet spectrum
  real(dp) function microjets_sum_as0 (pt)
    real(dp), intent(in) :: pt
    microjets_sum_as0 =  microjets_g_as0(pt) + microjets_q_as0(pt) 
  end function microjets_sum_as0
  real(dp) function microjets_sum_as1 (pt)
    real(dp), intent(in) :: pt
    microjets_sum_as1 = microjets_g_as1(pt) + microjets_q_as1(pt)    
  end function microjets_sum_as1
  real(dp) function microjets_sum_as2 (pt)
    real(dp), intent(in) :: pt
    microjets_sum_as2 = microjets_g_as2(pt) + microjets_q_as2(pt)    
  end function microjets_sum_as2
  real(dp) function microjets_sum_as3 (pt)
    real(dp), intent(in) :: pt
    microjets_sum_as3 = microjets_g_as3(pt) + microjets_q_as3(pt)    
  end function microjets_sum_as3
 

end module hoppet_convolutions
