!----------------------------------------------------------------------
!! A module to compute the pt binning, and integration of a continuous spectrum
!! over the defined pt bins.

module pt_bins
  use sub_defs_io
  use common_setup
  implicit none
  integer              :: nbin 
  real(dp),allocatable :: ptbin(:,:)

contains
  !----------------------------------------------------------------------
  ! Set up the ptbin array with bins in pt
  subroutine ptbin_setup(rts, ptmin)
    real(dp), intent(in) :: rts, ptmin
    integer :: ibin, iread
    if (log_val_opt("-bin")) then
       ! take spectrum from file, using option "-bin filename"
       ! number of bins needs to be specified with "-nbin nbins"
       iread = idev_open_opt("-bin")
       nbin = int_val_opt("-nbin",50)
       allocate(ptbin(1:nbin,1:3))
       do ibin = 1, nbin
          read(70,'(3es15.7)') ptbin(ibin,1), ptbin(ibin,2), ptbin(ibin,3)
       enddo
    elseif (log_val_opt("-alice")) then
       ! Take binning used in Alice data
       nbin = 10
       allocate(ptbin(1:nbin,1:3))
       ptbin( 1,1) =  20.0_dp
       ptbin( 2,1) =  25.0_dp
       ptbin( 3,1) =  30.0_dp
       ptbin( 4,1) =  35.0_dp
       ptbin( 5,1) =  40.0_dp
       ptbin( 6,1) =  50.0_dp
       ptbin( 7,1) =  60.0_dp
       ptbin( 8,1) =  70.0_dp
       ptbin( 9,1) =  85.0_dp
       ptbin(10,1) = 105.0_dp
       ptbin( 1,3) =  25.0_dp   
       ptbin( 2,3) =  30.0_dp
       ptbin( 3,3) =  35.0_dp
       ptbin( 4,3) =  40.0_dp
       ptbin( 5,3) =  50.0_dp
       ptbin( 6,3) =  60.0_dp
       ptbin( 7,3) =  70.0_dp
       ptbin( 8,3) =  85.0_dp
       ptbin( 9,3) = 105.0_dp
       ptbin(10,3) = 125.0_dp
       ptbin(: ,2) = half * (ptbin(:,1) + ptbin(:,3))
    else
       nbin = int_val_opt("-nbin",50)
       allocate(ptbin(1:nbin,1:3))
       do ibin = 1, nbin
          ptbin(ibin, 1) = ptmin * exp((ibin-1)/(1.0_dp*nbin)*log(rts/ptmin))
          ptbin(ibin, 2) = ptmin * exp((ibin-0.5_dp)/(1.0_dp*nbin)*log(rts/ptmin))
          ptbin(ibin, 3) = ptmin * exp(ibin/(1.0_dp*nbin)*log(rts/ptmin))
       end do
    endif
  end subroutine ptbin_setup

  !----------------------------------------------------------------------
  ! subroutine to integrate a spectrum defined by function spec_fct over
  ! given pt bins.
  !
  ! assumes uniform bin size in log pt
  ! 
  ! pt_bins is an (1:nb,1:3) array with
  !    bin1_min bin1_mid bin1_max
  !            ...
  !    binN_min binN_mid binN_max
  ! res is a (1:nb) array with
  !    spectrum_bin1
  !        ...
  !    spectrum_binN
  subroutine integrate_spec(res, spec_fct)
    use integration_dgauss
    real(dp), intent(out) :: res(1:nbin)
    integer               :: i
    interface
       real(dp) function spec_fct(pt)
         use types, only: dp
         implicit none
         real(dp), intent(in) :: pt
       end function
    end interface
    res = 0.0_dp
    do i = 1, nbin
       res(i) = dgauss(spec_fct,ptbin(i,1),ptbin(i,3),0.00001_dp)
       res(i) = res(i) / (ptbin(i,3) - ptbin(i,1))
    end do
  end subroutine integrate_spec
  
end module pt_bins
