/// \file example3.cc Example that prints out a large collection of observables
/// for a range of t values.
///
/// Usage:
/// 
/// \code
///     ./example3 [ -fixed-order [ -order O ] | -all-order [ -non-recursive ] [ -t T ] ] 
///                [ -q | -g ] [ -seed S ] [ -nev N ] [ -eps E ] [ -nbin B ]
///                [ -fcut F ] [ -nfilt M ] [ -nsteps NS ] [ -dt DT ] [ -output FN ] 
///                [ -ca CA ] [ -cf CF ] [ -tr TR] [ -nf NF ]
///               
/// \endcode
///
/// The code can be run to give fixed order results (-fixed-order),
/// specifically the coefficient of t^O, or all-order results
/// (-all-order) for a given value of t=T.
///
/// You can specify whether you start with a quark (default) or a
/// gluon with the -q and -g options.
///
/// Technical parameters (see default values in Input class):
///
/// - -non-recursive specifies an alternative all-order implementation
/// - -nev N         indicates that the program should generate N events (default 10^4)
/// - -eps E         specifies the value of the cutoff: E < z < 1-E.
/// - -seed S        specifies the random seed (default 1)
/// - -nbin B        indicates the number of bins for histograms
/// - -fcut F        specifies the treshold value F < z for trimming
/// - -nfilt M       specifies the number of hardest subjets for filtering
/// - -nsteps NS     specifies the number of values t to compute
/// - -dt DT         specifies the increment in t at each step
/// - -output FN     indicates the name of output file (default is to std::cout)
/// - -ca CA         specifies the value of CA in pgg
/// - -cf CF         specifies the value of CF in pqq
/// - -tr TR         specifies the value of TR in pqg
/// - -nf NF         specifies the value of NF in pqg
///

#include "Input.hh"
#include "NLOHist.hh"
#include "AverageAndError.hh"
#include "Observable.hh"
#include "Output.hh"
#include "Generator.hh"
#include "AllOrderGeneratorFlat.hh"
#include "FixedOrderGenerator.hh"
#include "AllOrderGeneratorRecursive.hh"

using namespace std;

int main(int argc, char ** argv) {
  Input in(argc,argv);
  cout << "# " << in.command << "\n# " << in.time << endl;
  Output output(in.filename,in.command,in.time);

  in.assert_all_options_used();
  Generator * generator;
  if (in.fixedorder) {
    in.nsteps=1;
    generator = new FixedOrderGenerator(in.seed,in.flav,in.order,
					in.ca,in.cf,in.tr,in.nf);
  }
  else if (in.nonrecursive) {
    generator = new AllOrderGeneratorFlat(in.seed,in.flav,in.eps,in.t,
					  in.ca,in.cf,in.tr,in.nf);
  }
  else {
    generator = new AllOrderGeneratorRecursive(in.seed,in.flav,in.eps,in.t,
					       in.ca,in.cf,in.tr,in.nf);
  }
  
  vector< map<string,AverageAndError> > averages(in.nsteps);

  // declare vector NLOHist, with histos[i][k] for t=t+i*dt and
  // k=0 for hardest, k=1 for inclusive spectrum
  vector< vector<NLOHist> > histos(in.nsteps, vector<NLOHist>(2));
  for(int i=0;i<in.nsteps;++i) {
    histos[i][0].declare("x max",0.0,1.+1e-9,in.nbin);
    histos[i][1].declare("x inclusive",0.0,1.+1e-9,in.nbin);
  }
  Parton::Flavour changeflav;
  if (in.flav==Parton::q) changeflav=Parton::g;
  if (in.flav==Parton::g) changeflav=Parton::q;
  FlavHardest changeflavhardest(changeflav);
  Filter filter(in.nfilt);
  Trim trim(in.fcut);

  // Loop over events
  for (int iev = 0; iev < in.nev; iev++) {
    // Loop from t=in.t to in.t+(in.nsteps-1)*in.dt
    for (int k=0;k<in.nsteps;++k) {
      // events may come in "groups" (like for a NLO program);
      // generate the next group
      generator->set_tmax(in.t+k*in.dt);
      generator->next_event_group();

      // get a reference to the next group of events
      const vector<Event> & events = generator->events();

      // calculate observables
      averages[k]["delta z"]   += average_over_group(events,deltaxmax);
      averages[k]["log z"]     += average_over_group(events,logxmax);
      averages[k]["log^2 z"]     += average_over_group(events,log2xmax);
      averages[k]["prob flavour change"]
	                       += average_over_group(events,changeflavhardest);
      averages[k]["z^6"]       += average_over_group(events,xmax6);
      averages[k]["trimmed z"] += average_over_group(events,trim);
      averages[k]["filtered z"]+= average_over_group(events,filter);
      add_group_to_histogram(histos[k][0],events,xmax);
      add_group_to_histogram(histos[k][1],events,xinclusive);      
    }
    for(int k=0;k<in.nsteps;++k) {
      for(vector<NLOHist>::iterator it=histos[k].begin();
	  it != histos[k].end(); ++it)
	it->collate_event();
    }
    // Write results when writing_step(iev)==true
    if(output.writing_step(iev)) {
      output.write(averages,histos,iev,in.t,in.dt,in.nsteps,string("t"));
      cout << "# Currently at " << iev << " out of "
	   << in.nev << " events." << endl;
    }
    
    // Reset to initial configuration for next event
    generator->reset();
  }
  output.write(averages,histos,in.nev,in.t,in.dt,in.nsteps,string("t"));
  cout << "# Simulation completed" << endl;
}

