!! -----------------------------------------------------
!
!  Compute and save the microjet spectrum. Uses
!   - common_setup
!   - parton_spect
!   - hoppet_convolutions
!   - pt_bins
!   
!! -----------------------------------------------------

program inclusive_spectrum
  use common_setup
  use sub_defs_io
  use hoppet_convolutions
  use parton_spect
  use pt_bins
  ! if using LHAPDF, rename a couple of hoppet functions which
  ! would otherwise conflict with LHAPDF 
  use hoppet_v1, EvolvePDF_hoppet => EvolvePDF, InitPDF_hoppet => InitPDF

  implicit none
  !! hold the initial pdf
  ! quantities needed for the jet spectrum
  real(dp), pointer    :: pts(:)
  real(dp)             :: ptmin, rts, xmur, xmuf, norm
  integer              :: ibin
  real(dp),allocatable :: temp1(:), temp2(:), temp3(:), temp4(:)
  logical           :: do_hoppet_conv, do_pt_bins
  !! hold results at some x, Q
  integer  :: idev, ipt

  if (log_val_opt("-out")) then
     idev = idev_open_opt("-out")
  else
     idev = 6
  end if
  hoppet_idev = idev

  ! allow for a user-controlled normalisation factor in order to get
  ! consistent results
  norm = dble_val_opt("-norm", one)
  
  ! turn off the hoppet evolution with -no-hoppet-conv
  do_hoppet_conv = .not.log_val_opt("-no-hoppet-conv")

  rts = dble_val_opt("-rts",14000.0_dp)
  ptmin = dble_val_opt("-ptmin",50.0_dp)
  ymax = log(rts/ptmin)  ! default for common setup

  ! integrate over pt bins with -pt-bins
  do_pt_bins = log_val_opt("-pt-bins")
  if (do_pt_bins) then
     ! set up the ptbin array (by default 50 equidistant bins in logspace)
     call ptbin_setup(rts, ptmin)
     allocate(temp1(nbin), temp2(nbin), temp3(nbin), temp4(nbin))
  endif

  ! basic setup
  call do_common_setup()
  call print_common_setup(idev)
  write(hoppet_idev,"(a,f5.3)") "# norm = ", norm
  
  ! get the pt's
  call AllocGridQuant(grid, pts)
  pts = rts * xValues(grid)

  ! set xmur and xmuf
  xmur = dble_val_opt("-xmur",1.0_dp)
  xmuf = dble_val_opt("-xmuf",1.0_dp)

  ! set up the parton spectrum using parton_spectrum.f90
  call setup_partonspec(pts, rts, ptmin, xmur, xmuf)
  parton_spectrum = parton_spectrum * norm
  
  if (do_hoppet_conv) then
     write(0,'(a)') 'Performing hoppet evolution from parton spectrum.'
     call hoppet_conv(pts, rts, parton_spectrum, xmur, spectrum_type)
  endif
  
  ! print the results  
  if (do_hoppet_conv.and.(.not.do_pt_bins)) then
     ! print result of hoppet evolution, not integrated in pt bins
     write(0,'(a)') 'Now writing the result of hoppet evolution to output file.'
     write(idev,'(a)') '# pt, gluon spectrum, quark spectrum, microjet spectrum'
     do ipt = grid%ny, 0, -1
        write(idev,'(f15.5,3es15.7)') pts(ipt), parton_spectrum(ipt,:), &
             & sum(microjet_spectrum(ipt,:,:))
     end do

     write(idev,'(a)') ''
     write(idev,'(a)') ''
     write(idev,'(a)') '# pt, microjet spectrum from gluon, microjet spectrum from quark'
     do ipt = grid%ny, 0, -1
        write(idev,'(f15.5,2es15.7)') pts(ipt), sum(microjet_spectrum(ipt,0,:)), &
             & sum(microjet_spectrum(ipt,1,:))
     end do

     write(idev,'(a)') ''
     write(idev,'(a)') ''
     write(idev,'(a,a)') '# pt, gluon microjet from gluon, quark microjet from gluon, ',&
          &            'gluon microjet from quark, quark microjet from quark'
     do ipt = grid%ny, 0, -1
        write(idev,'(f15.5,4es15.7)') pts(ipt), microjet_spectrum(ipt,0,0), &
        & microjet_spectrum(ipt,0,1), microjet_spectrum(ipt,1,0), microjet_spectrum(ipt,1,1)
     end do

     write(idev,'(a)') ''
     write(idev,'(a)') ''
     write(idev,'(a)') '# pt, microjet spectrum from gluon expanded in t'
     do ipt = grid%ny, 0, -1
        write(idev,'(f15.5,4es15.7)') pts(ipt), microjets_t(ipt,0,:)
     end do

     write(idev,'(a)') ''
     write(idev,'(a)') ''
     write(idev,'(a)') '# pt, microjet spectrum from quark expanded in t'
     do ipt = grid%ny, 0, -1
        write(idev,'(f15.5,4es15.7)') pts(ipt), microjets_t(ipt,1,:)
     end do

     write(idev,'(a)') ''
     write(idev,'(a)') ''
     write(idev,'(a)') '# pt, flavour-summed spectrum expanded in t'
     do ipt = grid%ny, 0, -1
        write(idev,'(f15.5,4es15.7)') pts(ipt), sum(microjets_t(ipt,:,:),dim=1)
     end do

     write(idev,'(a)') ''
     write(idev,'(a)') ''
     write(idev,'(a)') '# pt, microjet spectrum from gluon expanded in alphas'
     do ipt = grid%ny, 0, -1
        write(idev,'(f15.5,4es15.7)') pts(ipt), microjets_as(ipt,0,:)
     end do

     write(idev,'(a)') ''
     write(idev,'(a)') ''
     write(idev,'(a)') '# pt, microjet spectrum from quark expanded in alphas'
     do ipt = grid%ny, 0, -1
        write(idev,'(f15.5,4es15.7)') pts(ipt), microjets_as(ipt,1,:)
     end do

     write(idev,'(a)') ''
     write(idev,'(a)') ''
     write(idev,'(a)') '# pt, flavour-summed microjet spectrum expanded in alphas'
     do ipt = grid%ny, 0, -1
        write(idev,'(f15.5,4es15.7)') pts(ipt), sum(microjets_as(ipt,:,:),dim=1)
     end do
  !--------------------------------------------------
  elseif (do_hoppet_conv.and.do_pt_bins) then
     ! print result of hoppet evolution, in pt bins
     write(0,'(a)') 'Now writing the result of hoppet evolution to output file.'
     write(0,'(a)') 'Performing integration over pt bins.'
     call integrate_spec(temp1, spectrum_g)
     call integrate_spec(temp2, spectrum_q)
     call integrate_spec(temp3, microjet_spec_from_gluon)
     call integrate_spec(temp4, microjet_spec_from_quark)
     write(idev,'(a)') '# bin_min bin_mid bin_max gluon_spectrum quark_spectrum microjet_spectrum'
     do ibin = 1, nbin
        write(idev,'(3f15.5,3es15.7)') ptbin(ibin,:), temp1(ibin), &
             & temp2(ibin), temp3(ibin)+temp4(ibin)
     end do

     write(idev,'(a)') ''
     write(idev,'(a)') ''
     write(idev,'(a)') '# bin_min bin_mid bin_max microjets_gluon microjets_quark'
     do ibin = 1, nbin
        write(idev,'(3f15.5,2es15.7)') ptbin(ibin,:), temp3(ibin), &
             & temp4(ibin)
     end do
     call integrate_spec(temp1, gluon_microjet_spec_from_gluon)
     call integrate_spec(temp2, quark_microjet_spec_from_gluon)
     call integrate_spec(temp3, gluon_microjet_spec_from_quark)
     call integrate_spec(temp4, quark_microjet_spec_from_quark)
     write(idev,'(a)') ''
     write(idev,'(a)') ''
     write(idev,'(a,a)') '# bin_min bin_mid bin_max gluon_microjets_from_gluon ', &
          & 'quark_microjets_from_gluon gluon_microjets_from_quark quark_microjets_from_quark'
     do ibin = 1, nbin
        write(idev,'(3f15.5,4es15.7)') ptbin(ibin,:), temp1(ibin), temp2(ibin), &
             & temp3(ibin), temp4(ibin)
     end do

     call integrate_spec(temp1, microjets_g_t0)
     call integrate_spec(temp2, microjets_g_t1)
     call integrate_spec(temp3, microjets_g_t2)
     call integrate_spec(temp4, microjets_g_t3)
     write(idev,'(a)') ''
     write(idev,'(a)') ''
     write(idev,'(a)') '# bin_min bin_mid bin_max microjets_gluon_t'
     do ibin = 1, nbin
        write(idev,'(3f15.5,4es15.7)') ptbin(ibin,:), temp1(ibin), &
             & temp2(ibin), temp3(ibin), temp4(ibin)
     end do

     call integrate_spec(temp1, microjets_q_t0)
     call integrate_spec(temp2, microjets_q_t1)
     call integrate_spec(temp3, microjets_q_t2)
     call integrate_spec(temp4, microjets_q_t3)
     write(idev,'(a)') ''
     write(idev,'(a)') ''
     write(idev,'(a)') '# bin_min bin_mid bin_max microjets_quark_t'
     do ibin = 1, nbin
        write(idev,'(3f15.5,4es15.7)') ptbin(ibin,:), temp1(ibin), &
             & temp2(ibin), temp3(ibin), temp4(ibin)
     end do

     call integrate_spec(temp1, microjets_sum_t0)
     call integrate_spec(temp2, microjets_sum_t1)
     call integrate_spec(temp3, microjets_sum_t2)
     call integrate_spec(temp4, microjets_sum_t3)
     write(idev,'(a)') ''
     write(idev,'(a)') ''
     write(idev,'(a)') '# bin_min bin_mid bin_max microjets_sum_t'
     do ibin = 1, nbin
        write(idev,'(3f15.5,4es15.7)') ptbin(ibin,:), temp1(ibin), &
             & temp2(ibin), temp3(ibin), temp4(ibin)
     end do

     call integrate_spec(temp1, microjets_g_as0)
     call integrate_spec(temp2, microjets_g_as1)
     call integrate_spec(temp3, microjets_g_as2)
     call integrate_spec(temp4, microjets_g_as3)
     write(idev,'(a)') ''
     write(idev,'(a)') ''
     write(idev,'(a)') '# bin_min bin_mid bin_max microjets_gluon_as'
     do ibin = 1, nbin
        write(idev,'(3f15.5,4es15.7)') ptbin(ibin,:), temp1(ibin), &
             & temp2(ibin), temp3(ibin), temp4(ibin)
     end do

     call integrate_spec(temp1, microjets_q_as0)
     call integrate_spec(temp2, microjets_q_as1)
     call integrate_spec(temp3, microjets_q_as2)
     call integrate_spec(temp4, microjets_q_as3)
     write(idev,'(a)') ''
     write(idev,'(a)') ''
     write(idev,'(a)') '# bin_min bin_mid bin_max microjets_quark_as'
     do ibin = 1, nbin
        write(idev,'(3f15.5,4es15.7)') ptbin(ibin,:), temp1(ibin), &
             & temp2(ibin), temp3(ibin), temp4(ibin)
     end do

     call integrate_spec(temp1, microjets_sum_as0)
     call integrate_spec(temp2, microjets_sum_as1)
     call integrate_spec(temp3, microjets_sum_as2)
     call integrate_spec(temp4, microjets_sum_as3)
     write(idev,'(a)') ''
     write(idev,'(a)') ''
     write(idev,'(a)') '# bin_min bin_mid bin_max microjets_sum_as'
     do ibin = 1, nbin
        write(idev,'(3f15.5,4es15.7)') ptbin(ibin,:), temp1(ibin), &
             & temp2(ibin), temp3(ibin), temp4(ibin)
     end do
  !--------------------------------------------------
  elseif ((.not.do_hoppet_conv).and.do_pt_bins) then
     ! print spectrum in pt bins
     write(0,'(a)') 'Performing integration over pt bins.'
     call integrate_spec(temp1, spectrum_g)
     call integrate_spec(temp2, spectrum_q)
     write(idev,'(a)') '# bin_min bin_mid bin_max gluon_spectrum quark_spectrum'
     do ibin = 1, nbin
        write(idev,'(3f15.5,2es15.7)') ptbin(ibin, :), temp1(ibin), &
             & temp2(ibin)
     end do
  !--------------------------------------------------
  else
     ! print spectrum
     write(idev,'(a)') '# pt gluon_spectrum quark_spectrum'
     do ipt = grid%ny, 0, -1
        write(idev,'(f15.5,2es15.7)') pts(ipt), parton_spectrum(ipt,:)
     end do
  endif

  write(idev,'(a)') ''
  write(idev,'(a)') ''


  ! Some cleaning
  call common_delete()

  if (.not. CheckAllArgsUsed(0)) stop

contains


end program inclusive_spectrum


